@@ -272,11 +272,7 @@ namespace Nes
 					NES_DO_POKE(4015,0x4015,0x09);
 				}
 
-				if (cpu.GetModel() == CPU_DENDY)
-				{
-					ctrl = STATUS_NO_FRAME_IRQ;
-				}
-				else if (hard)
+				if (hard)
 				{
 					ctrl = STATUS_FRAME_IRQ_ENABLE;
 				}
@@ -289,10 +285,7 @@ namespace Nes
 			}
 			else
 			{
-				if (cpu.GetModel() == CPU_DENDY)
-					ctrl = STATUS_NO_FRAME_IRQ;
-				else
-					ctrl = STATUS_FRAME_IRQ_ENABLE;
+				ctrl = STATUS_FRAME_IRQ_ENABLE;
 			}
 		}
 
@@ -430,7 +423,7 @@ namespace Nes
 
 			square[0].UpdateSettings ( settings.muted ? 0 : settings.volumes[ Channel::APU_SQUARE1  ], rate, fixed );
 			square[1].UpdateSettings ( settings.muted ? 0 : settings.volumes[ Channel::APU_SQUARE2  ], rate, fixed );
-			triangle.UpdateSettings  ( settings.muted ? 0 : settings.volumes[ Channel::APU_TRIANGLE ], rate, fixed, cpu.GetModel() );
+			triangle.UpdateSettings  ( settings.muted ? 0 : settings.volumes[ Channel::APU_TRIANGLE ], rate, fixed );
 			noise.UpdateSettings     ( settings.muted ? 0 : settings.volumes[ Channel::APU_NOISE    ], rate, fixed );
 			dmc.UpdateSettings       ( settings.muted ? 0 : settings.volumes[ Channel::APU_DPCM     ] );
 
@@ -1588,13 +1581,10 @@ namespace Nes
 		#pragma optimize("s", on)
 		#endif
 
-		void Apu::Triangle::UpdateSettings(uint v,dword r,uint f,CpuModel model)
+		void Apu::Triangle::UpdateSettings(uint v,dword r,uint f)
 		{
 			Oscillator::UpdateSettings( r, f );
 
-			if (model == CPU_DENDY)
-				v = v * 85 / Channel::DEFAULT_VOLUME;
-
 			outputVolume = (v * Channel::OUTPUT_MUL + Channel::DEFAULT_VOLUME/2) / Channel::DEFAULT_VOLUME;
 			active = CanOutput();
 		}
@@ -1971,6 +1961,7 @@ namespace Nes
 			dma.buffered       = false;
 			dma.address        = 0xC000;
 			dma.buffer         = 0x00;
+			overclockingIsSafe = true;
 		}
 
 		Cycle Apu::Dmc::GetResetFrequency(CpuModel model)
@@ -2184,16 +2175,31 @@ namespace Nes
 		{
 			out.dac = data & 0x7F;
 			curSample = out.dac * outputVolume;
+
+			if (out.dac != 0)
+			{
+				overclockingIsSafe = false;
+			}
 		}
 
 		NST_SINGLE_CALL void Apu::Dmc::WriteReg2(const uint data)
 		{
 			regs.address = 0xC000 | (data << 6);
+
+			if (regs.address != 0)
+			{
+				overclockingIsSafe = true;
+			}
 		}
 
 		NST_SINGLE_CALL void Apu::Dmc::WriteReg3(const uint data)
 		{
 			regs.lengthCounter = (data << 4) + 1;
+
+			if (regs.lengthCounter != 0)
+			{
+				overclockingIsSafe = true;
+			}
 		}
 
 		NST_SINGLE_CALL bool Apu::Dmc::ClockDAC()
