@@ -108,6 +108,7 @@ namespace Nes
 		yuvMap (NULL)
 		{
 			cycles.one = PPU_RP2C02_CC;
+			overclocked = false;
 			PowerOff();
 		}
 
@@ -137,37 +138,28 @@ namespace Nes
 					cpu.Map( i+7 ).Set( this,               &Ppu::Peek_2007,                   &Ppu::Poke_2007 );
 				}
 
-				switch (model)
+				if (model == PPU_RC2C05_01 || model == PPU_RC2C05_04)
 				{
-					case PPU_RC2C05_01:
-					case PPU_RC2C05_02:
-					case PPU_RC2C05_03:
-					case PPU_RC2C05_04:
-
-						if (model == PPU_RC2C05_02)
-						{
-							for (uint i=0x2002; i < 0x4000; i += 0x8)
-								cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_02 );
-						}
-						else if (model == PPU_RC2C05_03)
-						{
-							for (uint i=0x2002; i < 0x4000; i += 0x8)
-								cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_03 );
-						}
-						else
-						{
-							for (uint i=0x2002; i < 0x4000; i += 0x8)
-								cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_01_04 );
-						}
-
-					case PPU_RC2C05_05:
-
-						for (uint i=0x2000; i < 0x4000; i += 0x8)
-						{
-							cpu.Map( i+0 ).Set( &Ppu::Poke_2001 );
-							cpu.Map( i+1 ).Set( &Ppu::Poke_2000 );
-						}
-						break;
+					for (uint i=0x2002; i < 0x4000; i += 0x8)
+						cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_01_04 );
+				}
+				else if (model == PPU_RC2C05_02)
+				{
+					for (uint i=0x2002; i < 0x4000; i += 0x8)
+						cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_02 );
+				}
+				else if (model == PPU_RC2C05_03)
+				{
+					for (uint i=0x2002; i < 0x4000; i += 0x8)
+						cpu.Map( i ).Set( &Ppu::Peek_2002_RC2C05_03 );
+				}
+				else if (model == PPU_RC2C05_05)
+				{
+					for (uint i=0x2000; i < 0x4000; i += 0x8)
+					{
+						cpu.Map( i+0 ).Set( &Ppu::Poke_2001 );
+						cpu.Map( i+1 ).Set( &Ppu::Poke_2000 );
+					}
 				}
 
 				cpu.Map( 0x4014U ).Set( this, &Ppu::Peek_4014, &Ppu::Poke_4014 );
@@ -248,6 +240,8 @@ namespace Nes
 			cycles.count = Cpu::CYCLE_MAX;
 
 			scanline = SCANLINE_VBLANK;
+			scanline_sleep = -1;
+			ssleep = -1;
 
 			io.address = 0;
 			io.pattern = 0;
@@ -310,7 +304,7 @@ namespace Nes
 		void Ppu::UpdatePalette()
 		{
 			for (uint i=0, c=Coloring(), e=Emphasis(); i < Palette::SIZE; ++i)
-				output.palette[i] = (rgbMap ? rgbMap[palette.ram[i] & uint(Palette::COLOR)] : palette.ram[i]) & c | e;
+				output.palette[i] = ((rgbMap ? rgbMap[palette.ram[i] & uint(Palette::COLOR)] : palette.ram[i]) & c) | e;
 		}
 
 		void Ppu::SaveState(State::Saver& state,const dword baseChunk) const
@@ -499,6 +493,8 @@ namespace Nes
 
 			Cycle frame;
 
+			scanline_sleep = -1;
+
 			switch (model)
 			{
 				case PPU_RP2C02:
@@ -507,6 +503,8 @@ namespace Nes
 
 				default:
 
+					ssleep = PPU_RP2C02_VSLEEP - 2;
+
 					if (cycles.hClock == HCLOCK_DUMMY)
 					{
 						cycles.vClock = PPU_RP2C02_HVINT / PPU_RP2C02_CC - HCLOCK_DUMMY;
@@ -523,6 +521,8 @@ namespace Nes
 
 				case PPU_RP2C07:
 
+					ssleep = PPU_RP2C07_VSLEEP - 2;
+
 					if (cycles.hClock == HCLOCK_DUMMY)
 					{
 						cycles.vClock = PPU_RP2C07_HVINT / PPU_RP2C07_CC - HCLOCK_DUMMY;
@@ -539,6 +539,8 @@ namespace Nes
 
 				case PPU_DENDY:
 
+					ssleep = PPU_DENDY_VSLEEP - 2;
+
 					if (cycles.hClock == HCLOCK_DUMMY)
 					{
 						cycles.vClock = PPU_DENDY_HVINT / PPU_DENDY_CC - HCLOCK_DUMMY;
@@ -554,6 +556,38 @@ namespace Nes
 					break;
 			}
 
+			if (overclocked)
+			{
+				Apu& audioSafeOverclock = cpu.GetApu();
+				if (audioSafeOverclock.GetOverclockSafety())
+				{
+					switch (model)
+					{
+						case PPU_RP2C02:
+						default:
+
+						cpu.SetOverclocking(true,PPU_RP2C02_HSYNC * PPU_RP2C02_VACTIVE);
+						break;
+
+						case PPU_RP2C07:
+
+						cpu.SetOverclocking(true,PPU_RP2C07_HSYNC * PPU_RP2C07_VACTIVE);
+						break;
+
+						case PPU_DENDY:
+
+						cpu.SetOverclocking(true,PPU_DENDY_HSYNC * PPU_DENDY_VACTIVE);
+						break;
+					}
+				}
+				else
+				{
+					cpu.SetOverclocking(false,0);
+				}
+
+				audioSafeOverclock.SetOverclockSafety(true);//overclocking is only safe if direct pcm audio has not been written for one frame
+			}
+
 			cpu.SetFrameCycles( frame );
 		}
 
@@ -800,12 +834,12 @@ namespace Nes
 					if (!map)
 					{
 						for (uint i=0; i < Palette::SIZE; ++i)
-							output.palette[i] = palette.ram[i] & ce[0] | ce[1];
+							output.palette[i] = (palette.ram[i] & ce[0]) | ce[1];
 					}
 					else
 					{
 						for (uint i=0; i < Palette::SIZE; ++i)
-							output.palette[i] = map[palette.ram[i] & Palette::COLOR] & ce[0] | ce[1];
+							output.palette[i] = (map[palette.ram[i] & Palette::COLOR] & ce[0]) | ce[1];
 					}
 				}
 			}
@@ -826,17 +860,17 @@ namespace Nes
 
 		NES_PEEK_A(Ppu,2002_RC2C05_01_04)
 		{
-			return NES_DO_PEEK(2002,address) & 0xC0 | 0x1B;
+			return (NES_DO_PEEK(2002,address) & 0xC0) | 0x1B;
 		}
 
 		NES_PEEK_A(Ppu,2002_RC2C05_02)
 		{
-			return NES_DO_PEEK(2002,address) & 0xC0 | 0x3D;
+			return (NES_DO_PEEK(2002,address) & 0xC0) | 0x3D;
 		}
 
 		NES_PEEK_A(Ppu,2002_RC2C05_03)
 		{
-			return NES_DO_PEEK(2002,address) & 0xC0 | 0x1C;
+			return (NES_DO_PEEK(2002,address) & 0xC0) | 0x1C;
 		}
 
 		NES_POKE_D(Ppu,2003)
@@ -957,7 +991,7 @@ namespace Nes
 			{
 				address &= 0x1F;
 
-				const uint final = (!rgbMap ? data : rgbMap[data & Palette::COLOR]) & Coloring() | Emphasis();
+				const uint final = ((!rgbMap ? data : rgbMap[data & Palette::COLOR]) & Coloring()) | Emphasis();
 
 				palette.ram[address] = data;
 				output.palette[address] = final;
@@ -1301,7 +1335,7 @@ namespace Nes
 				address = (regs.ctrl[0] & Regs::CTRL0_SP_OFFSET) << 9 | buffer[1] << 4;
 			}
 
-			return address | comparitor & Oam::XFINE;
+			return address | (comparitor & Oam::XFINE);
 		}
 
 		NST_FORCE_INLINE void Ppu::LoadSprite(const uint pattern0,const uint pattern1,const byte* const NST_RESTRICT buffer)
@@ -1418,7 +1452,410 @@ namespace Nes
 		{
 			NST_VERIFY( cycles.count != cycles.hClock );
 
-			if (regs.ctrl[1] & Regs::CTRL1_BG_SP_ENABLED)
+			if (scanline_sleep >= 0)
+			{
+				switch (cycles.hClock)
+				{
+					case 0:
+					case 1:
+					case 2:
+					case 3:
+					case 4:
+					case 5:
+					case 6:
+					case 7:
+					case 8:
+					case 9:
+					case 10:
+					case 11:
+					case 12:
+					case 13:
+					case 14:
+					case 15:
+					case 16:
+					case 17:
+					case 18:
+					case 19:
+					case 20:
+					case 21:
+					case 22:
+					case 23:
+					case 24:
+					case 25:
+					case 26:
+					case 27:
+					case 28:
+					case 29:
+					case 30:
+					case 31:
+					case 32:
+					case 33:
+					case 34:
+					case 35:
+					case 36:
+					case 37:
+					case 38:
+					case 39:
+					case 40:
+					case 41:
+					case 42:
+					case 43:
+					case 44:
+					case 45:
+					case 46:
+					case 47:
+					case 48:
+					case 49:
+					case 50:
+					case 51:
+					case 52:
+					case 53:
+					case 54:
+					case 55:
+					case 56:
+					case 57:
+					case 58:
+					case 59:
+					case 60:
+					case 61:
+					case 62:
+					case 63:
+					case 64:
+					case 65:
+					case 66:
+					case 67:
+					case 68:
+					case 69:
+					case 70:
+					case 71:
+					case 72:
+					case 73:
+					case 74:
+					case 75:
+					case 76:
+					case 77:
+					case 78:
+					case 79:
+					case 80:
+					case 81:
+					case 82:
+					case 83:
+					case 84:
+					case 85:
+					case 86:
+					case 87:
+					case 88:
+					case 89:
+					case 90:
+					case 91:
+					case 92:
+					case 93:
+					case 94:
+					case 95:
+					case 96:
+					case 97:
+					case 98:
+					case 99:
+					case 100:
+					case 101:
+					case 102:
+					case 103:
+					case 104:
+					case 105:
+					case 106:
+					case 107:
+					case 108:
+					case 109:
+					case 110:
+					case 111:
+					case 112:
+					case 113:
+					case 114:
+					case 115:
+					case 116:
+					case 117:
+					case 118:
+					case 119:
+					case 120:
+					case 121:
+					case 122:
+					case 123:
+					case 124:
+					case 125:
+					case 126:
+					case 127:
+					case 128:
+					case 129:
+					case 130:
+					case 131:
+					case 132:
+					case 133:
+					case 134:
+					case 135:
+					case 136:
+					case 137:
+					case 138:
+					case 139:
+					case 140:
+					case 141:
+					case 142:
+					case 143:
+					case 144:
+					case 145:
+					case 146:
+					case 147:
+					case 148:
+					case 149:
+					case 150:
+					case 151:
+					case 152:
+					case 153:
+					case 154:
+					case 155:
+					case 156:
+					case 157:
+					case 158:
+					case 159:
+					case 160:
+					case 161:
+					case 162:
+					case 163:
+					case 164:
+					case 165:
+					case 166:
+					case 167:
+					case 168:
+					case 169:
+					case 170:
+					case 171:
+					case 172:
+					case 173:
+					case 174:
+					case 175:
+					case 176:
+					case 177:
+					case 178:
+					case 179:
+					case 180:
+					case 181:
+					case 182:
+					case 183:
+					case 184:
+					case 185:
+					case 186:
+					case 187:
+					case 188:
+					case 189:
+					case 190:
+					case 191:
+					case 192:
+					case 193:
+					case 194:
+					case 195:
+					case 196:
+					case 197:
+					case 198:
+					case 199:
+					case 200:
+					case 201:
+					case 202:
+					case 203:
+					case 204:
+					case 205:
+					case 206:
+					case 207:
+					case 208:
+					case 209:
+					case 210:
+					case 211:
+					case 212:
+					case 213:
+					case 214:
+					case 215:
+					case 216:
+					case 217:
+					case 218:
+					case 219:
+					case 220:
+					case 221:
+					case 222:
+					case 223:
+					case 224:
+					case 225:
+					case 226:
+					case 227:
+					case 228:
+					case 229:
+					case 230:
+					case 231:
+					case 232:
+					case 233:
+					case 234:
+					case 235:
+					case 236:
+					case 237:
+					case 238:
+					case 239:
+					case 240:
+					case 241:
+					case 242:
+					case 243:
+					case 244:
+					case 245:
+					case 246:
+					case 247:
+					case 248:
+					case 249:
+					case 250:
+					case 251:
+					case 252:
+					case 253:
+					case 254:
+					case 255:
+					case 256:
+					case 257:
+					case 258:
+					case 260:
+					case 261:
+					case 262:
+					case 263:
+					case 264:
+					case 266:
+					case 267:
+					case 268:
+					case 269:
+					case 270:
+					case 271:
+					case 272:
+					case 273:
+					case 274:
+					case 275:
+					case 276:
+					case 277:
+					case 278:
+					case 279:
+					case 280:
+					case 281:
+					case 282:
+					case 283:
+					case 284:
+					case 285:
+					case 286:
+					case 287:
+					case 288:
+					case 289:
+					case 290:
+					case 291:
+					case 292:
+					case 293:
+					case 294:
+					case 295:
+					case 296:
+					case 297:
+					case 298:
+					case 299:
+					case 300:
+					case 301:
+					case 302:
+					case 303:
+					case 304:
+					case 305:
+					case 306:
+					case 307:
+					case 308:
+					case 309:
+					case 310:
+					case 311:
+					case 312:
+					case 313:
+					case 314:
+					case 315:
+					case 316:
+					case 317:
+					case 318:
+					case 319:
+					case 320:
+					case 321:
+					case 322:
+					case 323:
+					case 324:
+					case 325:
+					case 326:
+					case 327:
+					case 328:
+					case 329:
+					case 330:
+					case 331:
+					case 332:
+					case 333:
+					case 334:
+					case 335:
+					case 336:
+					case 337:
+					HActiveSleep:
+
+						cycles.hClock = 338;
+
+						if (cycles.count <= 338)
+							break;
+
+					case 338:
+
+						if (scanline_sleep++ != ssleep)
+						{
+							cycles.hClock = 0;
+							cycles.vClock += 341;
+
+							if (cycles.count <= 341)
+								break;
+
+							cycles.count -= 341;
+
+							goto HActiveSleep;
+						}
+						else
+						{
+							cycles.hClock = HCLOCK_VBLANK_0;
+
+							if (cycles.count <= HCLOCK_VBLANK_0)
+								break;
+						}
+
+					case HCLOCK_VBLANK_0:
+					VBlank0:
+
+						regs.status |= Regs::STATUS_VBLANKING;
+						cycles.hClock = HCLOCK_VBLANK_1;
+
+						if (cycles.count <= HCLOCK_VBLANK_1)
+							break;
+
+					case HCLOCK_VBLANK_1:
+					VBlank1:
+
+						regs.status = (regs.status & 0xFF) | (regs.status >> 1 & Regs::STATUS_VBLANK);
+						oam.visible = oam.output;
+						cycles.hClock = HCLOCK_VBLANK_2;
+
+						if (cycles.count <= HCLOCK_VBLANK_2)
+							break;
+
+					case HCLOCK_VBLANK_2:
+					VBlank2:
+
+						scanline_sleep = -1;
+
+						cycles.hClock = HCLOCK_DUMMY;
+						cycles.count = Cpu::CYCLE_MAX;
+						cycles.reset = 0;
+
+						if (regs.ctrl[0] & regs.status & Regs::CTRL0_NMI)
+							cpu.DoNMI( cpu.GetFrameCycles() );
+
+						return;
+				}
+			}
+			else if (regs.ctrl[1] & Regs::CTRL1_BG_SP_ENABLED)
 			{
 				switch (cycles.hClock)
 				{
@@ -2090,49 +2527,36 @@ namespace Nes
 						}
 						else
 						{
-							cycles.hClock = HCLOCK_POSTRENDER;
+							if (ssleep >= 0)
+							{
+								scanline_sleep = 0;
+								cycles.hClock = 0;
+								cycles.vClock += 341;
 
-							if (cycles.count <= HCLOCK_POSTRENDER)
-								break;
+								if (cycles.count <= 341)
+									break;
+
+								cycles.count -= 341;
+
+								goto HActiveSleep;
+							}
+							else
+							{
+								cycles.hClock = HCLOCK_VBLANK_0;
+
+								if (cycles.count <= HCLOCK_VBLANK_0)
+									break;
+							}
 						}
 
-					case HCLOCK_POSTRENDER:
-						UpdateAddressLine(scroll.address & 0x3fff);
-						cycles.hClock = HCLOCK_VBLANK_0;
-
-						if (cycles.count <= HCLOCK_VBLANK_0)
-							break;
-
 					case HCLOCK_VBLANK_0:
-					VBlank0:
-
-						regs.status |= Regs::STATUS_VBLANKING;
-						cycles.hClock = HCLOCK_VBLANK_1;
-
-						if (cycles.count <= HCLOCK_VBLANK_1)
-							break;
+						goto VBlank0;
 
 					case HCLOCK_VBLANK_1:
-					VBlank1:
-
-						regs.status = (regs.status & 0xFF) | (regs.status >> 1 & Regs::STATUS_VBLANK);
-						oam.visible = oam.output;
-						cycles.hClock = HCLOCK_VBLANK_2;
-
-						if (cycles.count <= HCLOCK_VBLANK_2)
-							break;
+						goto VBlank1;
 
 					case HCLOCK_VBLANK_2:
-					VBlank2:
-
-						cycles.hClock = HCLOCK_DUMMY;
-						cycles.count = Cpu::CYCLE_MAX;
-						cycles.reset = 0;
-
-						if (regs.ctrl[0] & regs.status & Regs::CTRL0_NMI)
-							cpu.DoNMI( cpu.GetFrameCycles() );
-
-						return;
+						goto VBlank2;
 
 					case HCLOCK_BOOT:
 						goto Boot;
@@ -2807,12 +3231,29 @@ namespace Nes
 
 							goto HActiveOff;
 						}
-						else
-						{
-							cycles.hClock = HCLOCK_VBLANK_0;
+ 						else
+ 						{
+							if (ssleep >= 0)
+							{
+								scanline_sleep = 0;
 
-							if (cycles.count <= HCLOCK_VBLANK_0)
-								break;
+								cycles.vClock += 341;
+								cycles.hClock = 0;
+
+								if (cycles.count <= 341)
+									break;
+
+								cycles.count -= 341;
+
+								goto HActiveSleep;
+							}
+							else
+							{
+								cycles.hClock = HCLOCK_VBLANK_0;
+
+								if (cycles.count <= HCLOCK_VBLANK_0)
+									break;
+							}
 						}
 
 					case HCLOCK_VBLANK_0:
